@@ -584,7 +584,7 @@ namespace dlib
                 info->radius = 0;
             }
             info->best_objective_value = y;
-            info->best_x = std::move(req.x);
+            info->best_x = req.x;
         }
     }
 
@@ -623,11 +623,18 @@ namespace dlib
     {
         DLIB_CASSERT(functions_.size() > 0);
         DLIB_CASSERT(functions_.size() == initial_function_evals.size());
-        DLIB_CASSERT(relative_noise_magnitude >= 0);
+        DLIB_CASSERT(relative_noise_magnitude_ >= 0);
         relative_noise_magnitude = relative_noise_magnitude_;
         for (size_t i = 0; i < initial_function_evals.size(); ++i)
         {
             functions[i]->ub = upper_bound_function(initial_function_evals[i], relative_noise_magnitude);
+
+            if (initial_function_evals[i].size() != 0)
+            {
+                auto best = max_scoring_element(initial_function_evals[i], [](const function_evaluation& e) { return e.y; }).first;
+                functions[i]->best_objective_value = best.y;
+                functions[i]->best_x = best.x;
+            }
         }
     }
 
@@ -889,7 +896,6 @@ namespace dlib
         size_t num
     )
     {
-        DLIB_CASSERT(0 <= num);
         num_random_samples = num;
     }
 
