@@ -67,7 +67,7 @@ namespace dlib
 //! \return modified string ``s'' with spaces trimmed from left
     inline std::string& triml(std::string& s)
     {
-        int pos(0);
+        std::string::size_type pos(0);
         for ( ; s[pos] == ' ' || s[pos] == '\t' || s[pos] == '\r' || s[pos] == '\n' ; ++pos );
         s.erase(0, pos);
         return s;
@@ -78,7 +78,7 @@ namespace dlib
 //! \return modified string ``s'' with spaces trimmed from right
     inline std::string& trimr(std::string& s)
     {
-        int pos(s.size());
+        std::string::size_type pos(s.size());
         for ( ; pos && (s[pos-1] == ' ' || s[pos-1] == '\t' || s[pos-1] == '\r' || s[pos-1] == '\n') ; --pos );
         s.erase(pos, s.size()-pos);
         return s;
@@ -119,7 +119,7 @@ namespace dlib
 #ifdef __WXMSW__
         ::ltoa(header_value, buf, 10);
 #else
-        sprintf(buf, "%ld", header_value);
+        snprintf(buf, sizeof(buf), "%ld", header_value);
 #endif
         set_header(header_name, buf);
     }
@@ -154,7 +154,7 @@ namespace dlib
 #ifdef __WXMSW__
         ::ltoa(cookie_value, buf, 10);
 #else
-        sprintf(buf, "%ld", cookie_value);
+        snprintf(buf, sizeof(buf), "%ld", cookie_value);
 #endif
         set_cookie(cookie_name, buf);
     }
