@@ -37,6 +37,7 @@ namespace dlib
             _num_trees_per_cascade_level = 500;
             _nu = 0.1;
             _oversampling_amount = 20;
+            _oversampling_translation_jitter = 0;
             _feature_pool_size = 400;
             _lambda = 0.1;
             _num_test_splits = 20;
@@ -116,6 +117,7 @@ namespace dlib
 
         unsigned long get_oversampling_amount (
         ) const { return _oversampling_amount; }
+
         void set_oversampling_amount (
             unsigned long amount
         )
@@ -129,6 +131,22 @@ namespace dlib
             _oversampling_amount = amount;
         }
 
+        double get_oversampling_translation_jitter (
+        ) const { return _oversampling_translation_jitter; }
+
+        void set_oversampling_translation_jitter (
+            double amount
+        )
+        {
+            DLIB_CASSERT(amount >= 0, 
+                "\t void shape_predictor_trainer::set_oversampling_translation_jitter()"
+                << "\n\t Invalid inputs were given to this function. "
+                << "\n\t amount: " << amount 
+            );
+
+            _oversampling_translation_jitter = amount;
+        }
+
         unsigned long get_feature_pool_size (
         ) const { return _feature_pool_size; }
         void set_feature_pool_size (
@@ -328,7 +346,7 @@ namespace dlib
             }
 
             if (_verbose)
-                std::cout << "Training complete                          " << std::endl;
+                std::cout << "\nTraining complete" << std::endl;
 
             return shape_predictor(initial_shape, forests, pixel_coordinates);
         }
@@ -706,6 +724,18 @@ namespace dlib
                         hits += alpha*samples[rand_idx].present;
                     }
                     samples[i].current_shape = pointwise_multiply(samples[i].current_shape, reciprocal(hits));
+
+                    if (_oversampling_translation_jitter != 0)
+                    {
+                        dpoint off;
+                        off.x() = rnd.get_double_in_range(-_oversampling_translation_jitter,_oversampling_translation_jitter);
+                        off.y() = rnd.get_double_in_range(-_oversampling_translation_jitter,_oversampling_translation_jitter);
+                        for (long j = 0; j < samples[i].current_shape.size()/2; ++j)
+                        {
+                            samples[i].current_shape(2*j) += off.x();
+                            samples[i].current_shape(2*j+1) += off.y();
+                        }
+                    }
                 }
 
             }
@@ -795,6 +825,7 @@ namespace dlib
         bool _verbose;
         unsigned long _num_threads;
         padding_mode_t _padding_mode;
+        double _oversampling_translation_jitter;
     };
 
 // ----------------------------------------------------------------------------------------
