@@ -631,6 +631,30 @@ namespace
                 3,3,3;
             DLIB_TEST(upperbound(m,3) == M);
         }
+        
+        {
+            matrix<double,9,5> A = randm(9,5);
+            matrix<double> B = A;
+            
+            orthogonalize(A);
+            orthogonalize(B);
+            
+            DLIB_TEST(equal(A,B));
+        }
+
+        {
+            matrix<double,9,5> A = randm(9,5);
+            matrix<double,9,5> B = randm(9,5);
+            matrix<double,9,5> C = pointwise_pow(A, B);
+
+            for (long r = 0; r < C.nr(); ++r)
+            {
+                for (long c = 0; c < C.nc(); ++c)
+                {
+                    DLIB_TEST(C(r, c) == std::pow(A(r, c), B(r, c)));
+                }
+            }
+        }
     }
 
 
@@ -784,9 +808,9 @@ namespace
         DLIB_TEST(temp3 == temp);
 
 
+        dlib::rand rnd;
         for (int i = 0; i < 3; ++i)
         {
-            dlib::rand rnd;
             matrix<complex<int> > a, b;
             a = complex_matrix(matrix_cast<int>(round(20*randm(2,7,rnd))), 
                                matrix_cast<int>(round(20*randm(2,7,rnd))));
@@ -797,6 +821,20 @@ namespace
             DLIB_TEST(xcorr_valid(a,b) == conv_valid(a, flip(conj(b))));
             DLIB_TEST(xcorr_same(a,b)  == conv_same(a, flip(conj(b))));
         }
+
+
+        for (int i = 0; i < 30; ++i)
+        {
+            auto nr1 = rnd.get_integer_in_range(1,30);
+            auto nc1 = rnd.get_integer_in_range(1,30);
+            auto nr2 = rnd.get_integer_in_range(1,30);
+            auto nc2 = rnd.get_integer_in_range(1,30);
+            matrix<double> a, b;
+            a = randm(nr1,nc1,rnd);
+            b = randm(nr2,nc2,rnd);
+
+            DLIB_TEST(max(abs(xcorr(a,b) - xcorr_fft(a,b))) < 1e-12);
+        }
     }
 
     void test_complex()
