@@ -7,11 +7,13 @@
 #include <cstdlib>
 #include <ctime>
 #include <dlib/statistics.h>
+#include <dlib/statistics/running_gradient.h>
 #include <dlib/rand.h>
 #include <dlib/svm.h>
 #include <algorithm>
 #include <dlib/matrix.h>
 #include <cmath>
+#include <random>
 
 #include "tester.h"
 
@@ -403,6 +405,8 @@ namespace
 
             string str = "DlibRocks";
 
+            std::minstd_rand shuffle_random;
+
             for(int j = 0; j<5 ; j++)
             {
                 matrix<double,1,100000> dat;
@@ -421,7 +425,7 @@ namespace
                 double exkurdenom = 0;
                 double unbi_exkur = 0;
 
-                random_shuffle(str.begin(), str.end());
+                std::shuffle(str.begin(), str.end(), shuffle_random);
                 rnum.set_seed(str);
 
                 for(int i = 0; i<n; i++)
@@ -801,6 +805,40 @@ namespace
             DLIB_TEST(equal_error_rate(vals2, vals1).first == 1);
         }
 
+        void test_equal_error_rate() 
+        {
+            auto result = equal_error_rate({}, {});
+            DLIB_TEST(result.first == 0);
+            DLIB_TEST(result.second == 0);
+
+            // no error case
+            result = equal_error_rate({1,1,1}, {2,2,2});
+            DLIB_TEST_MSG(result.first == 0, result.first);
+            DLIB_TEST_MSG(result.second == 2, result.second);
+
+            // max error case
+            result = equal_error_rate({2,2,2}, {1,1,1});
+            DLIB_TEST_MSG(result.first == 1, result.first);
+            DLIB_TEST_MSG(result.second == 2, result.second);
+            // Another way to have max error
+            result = equal_error_rate({1,1,1}, {1,1,1});
+            DLIB_TEST_MSG(result.second == 1, result.second);
+            DLIB_TEST_MSG(result.first == 1, result.first);
+
+            // wildly unbalanced  
+            result = equal_error_rate({}, {1,1,1});
+            DLIB_TEST_MSG(result.first == 0, result.first);
+
+            // wildly unbalanced  
+            result = equal_error_rate({1,1,1}, {});
+            DLIB_TEST_MSG(result.first == 0, result.first);
+
+            // 25% error case   
+            result = equal_error_rate({1,1,1,3}, {2, 2, 0, 2});
+            DLIB_TEST_MSG(result.first == 0.25, result.first);
+            DLIB_TEST_MSG(result.second == 2, result.second);
+        }
+
         void test_running_stats_decayed()
         {
             print_spinner();
@@ -866,6 +904,14 @@ namespace
             }
         }
 
+        void test_probability_values_are_increasing() {
+            DLIB_TEST(probability_values_are_increasing(std::vector<double>{1,2,3,4,5,6,7,8}) > 0.99);
+            DLIB_TEST(probability_values_are_increasing(std::vector<double>{8,7,6,5,4,4,3,2}) < 0.01);
+            DLIB_TEST(probability_values_are_increasing_robust(std::vector<double>{1,2,3,4,5,6,7,8}) > 0.99);
+            DLIB_TEST(probability_values_are_increasing_robust(std::vector<double>{8,7,6,5,4,4,3,2}) < 0.01);
+            DLIB_TEST(probability_values_are_increasing(std::vector<double>{1,2,1e10,3,4,5,6,7,8}) < 0.3);
+            DLIB_TEST(probability_values_are_increasing_robust(std::vector<double>{1,2,1e100,3,4,5,6,7,8}) > 0.99);
+        }
 
         void test_event_corr()
         {
@@ -907,6 +953,8 @@ namespace
             test_event_corr();
             test_running_stats_decayed();
             test_running_scalar_covariance_decayed();
+            test_equal_error_rate();
+            test_probability_values_are_increasing();
         }
     } a;
 
