@@ -20,7 +20,7 @@ numpy_image<rgb_pixel> load_rgb_image (const std::string &path)
     return img; 
 }
 
-numpy_image<rgb_pixel> load_rgb_alpha_image (const std::string &path)
+numpy_image<rgb_alpha_pixel> load_rgb_alpha_image (const std::string &path)
 {
     numpy_image<rgb_alpha_pixel> img;
     load_image(img, path);
@@ -166,6 +166,11 @@ void bind_numpy_returns(py::module &m)
 	py::arg("filename")
     );
 
+    m.def("load_rgb_alpha_image", &load_rgb_alpha_image,
+	"Takes a path and returns a numpy array (RGBA) containing the image",
+	py::arg("filename")
+    );
+
     m.def("load_grayscale_image", &load_grayscale_image, 
 	"Takes a path and returns a numpy array containing the image, as an 8bit grayscale image.",
 	py::arg("filename")
