@@ -41,8 +41,8 @@ point center(const rect_type& r) { return center(r); }
 template <typename rect_type>
 point dcenter(const rect_type& r) { return dcenter(r); }
 
-template <typename rect_type>
-bool contains(const rect_type& r, const point& p) { return r.contains(p); }
+template <typename rect_type, typename ptype>
+bool contains(const rect_type& r, const ptype& p) { return r.contains(p); }
 
 template <typename rect_type>
 bool contains_xy(const rect_type& r, const long x, const long y) { return r.contains(point(x, y)); }
@@ -87,29 +87,6 @@ string print_rect_filter(const rect_filter& r)
 }
 
 
-rectangle add_point_to_rect(const rectangle& r, const point& p)
-{
-    return r + p;
-}
-
-rectangle add_rect_to_rect(const rectangle& r, const rectangle& p)
-{
-    return r + p;
-}
-
-rectangle& iadd_point_to_rect(rectangle& r, const point& p)
-{
-    r += p;
-    return r;
-}
-
-rectangle& iadd_rect_to_rect(rectangle& r, const rectangle& p)
-{
-    r += p;
-    return r;
-}
-
-
 
 // ----------------------------------------------------------------------------------------
 
@@ -119,6 +96,8 @@ void bind_rectangles(py::module& m)
     typedef rectangle type;
     py::class_<type>(m, "rectangle", "This object represents a rectangular area of an image.")
         .def(py::init<long,long,long,long>(), py::arg("left"),py::arg("top"),py::arg("right"),py::arg("bottom"))
+        .def(py::init<drectangle>(), py::arg("rect"))
+        .def(py::init<rectangle>(), py::arg("rect"))
         .def(py::init())
         .def("area",   &::area)
         .def("left",   &::left)
@@ -127,19 +106,24 @@ void bind_rectangles(py::module& m)
         .def("bottom", &::bottom)
         .def("width",  &::width)
         .def("height", &::height)
+        .def("tl_corner", &type::tl_corner, "Returns the top left corner of the rectangle.")
+        .def("tr_corner", &type::tr_corner, "Returns the top right corner of the rectangle.")
+        .def("bl_corner", &type::bl_corner, "Returns the bottom left corner of the rectangle.")
+        .def("br_corner", &type::br_corner, "Returns the bottom right corner of the rectangle.")
         .def("is_empty", &::is_empty<type>)
         .def("center", &::center<type>)
         .def("dcenter", &::dcenter<type>)
-        .def("contains", &::contains<type>, py::arg("point"))
+        .def("contains", &::contains<type,point>, py::arg("point"))
+        .def("contains", &::contains<type,dpoint>, py::arg("point"))
         .def("contains", &::contains_xy<type>, py::arg("x"), py::arg("y"))
         .def("contains", &::contains_rec<type>, py::arg("rectangle"))
         .def("intersect", &::intersect<type>, py::arg("rectangle"))
         .def("__str__", &::print_rectangle_str<type>)
         .def("__repr__", &::print_rectangle_repr)
-        .def("__add__", &::add_point_to_rect)
-        .def("__add__", &::add_rect_to_rect)
-        .def("__iadd__", &::iadd_point_to_rect)
-        .def("__iadd__", &::iadd_rect_to_rect)
+        .def(py::self += point())
+        .def(py::self + point())
+        .def(py::self += rectangle())
+        .def(py::self + rectangle())
         .def(py::self == py::self)
         .def(py::self != py::self)
         .def(py::pickle(&getstate<type>, &setstate<type>));
@@ -148,6 +132,9 @@ void bind_rectangles(py::module& m)
     typedef drectangle type;
     py::class_<type>(m, "drectangle", "This object represents a rectangular area of an image with floating point coordinates.")
         .def(py::init<double,double,double,double>(), py::arg("left"), py::arg("top"), py::arg("right"), py::arg("bottom"))
+        .def(py::init<rectangle>(), py::arg("rect"))
+        .def(py::init<drectangle>(), py::arg("rect"))
+        .def(py::init<>())
         .def("area",   &::darea)
         .def("left",   &::dleft)
         .def("top",    &::dtop)
@@ -158,7 +145,12 @@ void bind_rectangles(py::module& m)
         .def("is_empty", &::is_empty<type>)
         .def("center", &::center<type>)
         .def("dcenter", &::dcenter<type>)
-        .def("contains", &::contains<type>, py::arg("point"))
+        .def("tl_corner", &type::tl_corner, "Returns the top left corner of the rectangle.")
+        .def("tr_corner", &type::tr_corner, "Returns the top right corner of the rectangle.")
+        .def("bl_corner", &type::bl_corner, "Returns the bottom left corner of the rectangle.")
+        .def("br_corner", &type::br_corner, "Returns the bottom right corner of the rectangle.")
+        .def("contains", &::contains<type,point>, py::arg("point"))
+        .def("contains", &::contains<type,dpoint>, py::arg("point"))
         .def("contains", &::contains_xy<type>, py::arg("x"), py::arg("y"))
         .def("contains", &::contains_rec<type>, py::arg("rectangle"))
         .def("intersect", &::intersect<type>, py::arg("rectangle"))
@@ -249,6 +241,7 @@ ensures \n\
     {
     typedef std::vector<rectangle> type;
     py::bind_vector<type>(m, "rectangles", "An array of rectangle objects.")
+        .def(py::init<size_t>(), py::arg("initial_size"))
         .def("clear", &type::clear)
         .def("resize", resize<type>)
         .def("extend", extend_vector_with_python_list<rectangle>)
@@ -258,11 +251,75 @@ ensures \n\
     {
     typedef std::vector<std::vector<rectangle>> type;
     py::bind_vector<type>(m, "rectangless", "An array of arrays of rectangle objects.")
+        .def(py::init<size_t>(), py::arg("initial_size"))
         .def("clear", &type::clear)
         .def("resize", resize<type>)
         .def("extend", extend_vector_with_python_list<rectangle>)
         .def(py::pickle(&getstate<type>, &setstate<type>));
     }
+
+    m.def("translate_rect", [](const rectangle& rect, const point& p){return translate_rect(rect,p);},
+" returns rectangle(rect.left()+p.x, rect.top()+p.y, rect.right()+p.x, rect.bottom()+p.y) \n\
+  (i.e. moves the location of the rectangle but doesn't change its shape)",
+        py::arg("rect"), py::arg("p"));
+
+    m.def("translate_rect", [](const drectangle& rect, const point& p){return translate_rect(rect,p);},
+" returns rectangle(rect.left()+p.x, rect.top()+p.y, rect.right()+p.x, rect.bottom()+p.y) \n\
+  (i.e. moves the location of the rectangle but doesn't change its shape)",
+        py::arg("rect"), py::arg("p"));
+
+    m.def("translate_rect", [](const rectangle& rect, const dpoint& p){return translate_rect(rect,point(p));},
+" returns rectangle(rect.left()+p.x, rect.top()+p.y, rect.right()+p.x, rect.bottom()+p.y) \n\
+  (i.e. moves the location of the rectangle but doesn't change its shape)",
+        py::arg("rect"), py::arg("p"));
+
+    m.def("translate_rect", [](const drectangle& rect, const dpoint& p){return translate_rect(rect,p);},
+" returns rectangle(rect.left()+p.x, rect.top()+p.y, rect.right()+p.x, rect.bottom()+p.y) \n\
+  (i.e. moves the location of the rectangle but doesn't change its shape)",
+        py::arg("rect"), py::arg("p"));
+
+
+
+    m.def("shrink_rect", [](const rectangle& rect, long num){return shrink_rect(rect,num);},
+" returns rectangle(rect.left()+num, rect.top()+num, rect.right()-num, rect.bottom()-num) \n\
+  (i.e. shrinks the given rectangle by shrinking its border by num)",
+        py::arg("rect"), py::arg("num"));
+
+    m.def("grow_rect", [](const rectangle& rect, long num){return grow_rect(rect,num);},
+"- return shrink_rect(rect, -num) \n\
+  (i.e. grows the given rectangle by expanding its border by num)",
+        py::arg("rect"), py::arg("num"));
+
+    m.def("scale_rect", [](const rectangle& rect, double scale){return scale_rect(rect,scale);},
+"- return scale_rect(rect, scale) \n\
+(i.e. resizes the given rectangle by a scale factor)",
+        py::arg("rect"), py::arg("scale"));
+
+    m.def("centered_rect", [](const point& p, unsigned long width, unsigned long height) {
+        return centered_rect(p, width, height); },
+        py::arg("p"), py::arg("width"), py::arg("height"));
+
+    m.def("centered_rects", [](const std::vector<point>& p, unsigned long width, unsigned long height) {
+        return centered_rects(p, width, height); },
+        py::arg("pts"), py::arg("width"), py::arg("height"));
+
+    m.def("centered_rect", [](const dpoint& p, unsigned long width, unsigned long height) {
+        return centered_rect(p, width, height); },
+        py::arg("p"), py::arg("width"), py::arg("height"));
+
+    m.def("centered_rect", [](const rectangle& rect, unsigned long width, unsigned long height) {
+        return centered_rect(rect, width, height); },
+        py::arg("rect"), py::arg("width"), py::arg("height"));
+
+    m.def("centered_rect", [](const drectangle& rect, unsigned long width, unsigned long height) {
+        return centered_rect(rect, width, height); },
+        py::arg("rect"), py::arg("width"), py::arg("height"));
+
+
+    m.def("center", [](const rectangle& rect){return center(rect); }, py::arg("rect"),
+        "    returns the center of the given rectangle");
+    m.def("center", [](const drectangle& rect){return center(rect); }, py::arg("rect"),
+        "    returns the center of the given rectangle");
 }
 
 // ----------------------------------------------------------------------------------------
